// can_api.cpp: Implementierungsdatei
//

// ------------------------------------------------------------------
// INCLUDE's fr Visual C++ 6
#include "stdafx.h"
#include <winioctl.h>
#include <stdlib.h>
#include <malloc.h>
#include <CanPar.h>
#include "can_api.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
// ------------------------------------------------------------------

CRITICAL_SECTION BusLoadCriticalSection; 

#ifdef __cplusplus
extern "C" {            /* C Deklarationen fr C++ */
#endif  /* __cplusplus */


static HIGHLOW Div64To32(HIGHLOW hl, ULONG d, ULONG n)
{
	unsigned long h,l;
	unsigned long a[4], b[4], c[4];
	HIGHLOW ret;

	h = hl.high;
	l = hl.low;

	a[0] = (h & 0xffff0000)>>16;
	a[1] = h & 0x0000ffff;
	a[2] = (l & 0xffff0000)>>16;
	a[3] = l & 0x0000ffff;

	b[0] = a[0] / d;
	c[0] = a[0] % d;
	a[1] = (a[1] + (c[0] << 16));
	b[1] = a[1] / d;
	c[1] = a[1] % d;

	a[2] = (a[2] + (c[1] << 16));
	b[2] = a[2] / d;
	c[2] = a[2] % d;
	
	a[3] = (a[3] + (c[2] << 16));
	b[3] = a[3] / d;
	c[3] = a[3] % d;


	ret.high  = (b[0] & 0x0000ffff) << 16;
	ret.high |= (b[1] & 0x0000ffff);
	ret.low   = (b[2] & 0x0000ffff) << 16;
	ret.low  |= (b[3] & 0x0000ffff);

	ret.low -= n;

	return ret;
}

static DWORD WINAPI canSenderThread(LPVOID lpThreadParam)
{
	struct can_api_send_thread *n;
	n = (struct can_api_send_thread *)lpThreadParam;
	
	while(!n->stop)
	{
		if(canTransmitDataByISR(n->pMain, &n->isrCAN))
		{
			Sleep(100);
		}
		if (n->dwDelay > 0) Sleep(n->dwDelay);
	}
	free(n);
	return 0;
}

static DWORD WINAPI canBackgroundThread(LPVOID lpThreadParam)
{
	struct can_api *pC;
	OVERLAPPED ovl;
	DWORD dwRet, dwErr;
	struct ISRInfo info;
	BOOL loop=TRUE;

	#ifdef TRACE
	TRACE("canBackgroundThread\n");
	#endif
	pC = (struct can_api *)lpThreadParam;
	memset(&ovl,0,sizeof(ovl));
	while(loop && !pC->iStopThread)
	{
		ovl.hEvent = pC->hSignal;
		
		// Bereit fr Datenempfang (vom Treiber)
		dwRet = DeviceIoControl(pC->hDevice, IOCTL_IO_7,NULL, NULL, NULL, 0, &dwRet, &ovl);
		dwErr = GetLastError();
		#ifdef TRACE
		TRACE("Nach DeviceIoControl 0x%x 0x%x\n",dwRet,dwErr);
		#endif
		if (!dwRet && (dwErr != ERROR_IO_PENDING))
		{	
			MessageBox(pC->hWnd,"Can't initialize CALLBACK function","Error",MB_OK|MB_ICONSTOP);
			break;
		}					

		// Wenn die Bereitschaftsmeldung geklappt hat -> Warte auf Benachrichtigung aus dem Treiber
		WaitForSingleObjectEx(pC->hSignal, INFINITE, TRUE);

		// Das Signal wurde gesetzt (kann aber auch aus dem Hauptprogramm heraus erfolgen)
		#ifdef TRACE
		TRACE("WaitForSingleObjectEx\n");
		#endif

		// Setze Signal zurck
		ResetEvent(pC->hSignal);

		// Thread beenden ?
		if (pC->iStopThread) 
		{
			// Ja -> Breche laufende Operation ab und beende die Endlosschleife und damit den Thread
			//CancelIo(pC->hDevice);
			break;
		}

		
		// Das Signal wurde im Treiber gesetzt -> Hole jetzt die Daten ab
		do
		{
			// Daten werden in einer Schleife solange abgeholt, bis:
			//  - keine Daten mehr vorhanden sind (info.isrNummer <> 0)
			//  - ein Fehler bei DeviceIOControl() auftrat
			//  - DeviceIOControl() keine (d.h. auch keine INFO-Struktur) zurckgab
			if (!DeviceIoControl(pC->hDevice, IOCTL_IO_13, NULL, 0, &info, sizeof(info), &dwRet, NULL))
			{
				#ifdef TRACE
				TRACE("Fehler nach IO_13\n");
				#endif
				loop = FALSE;
				break;
			}
			if (!dwRet)
			{
				#ifdef TRACE
				TRACE("IO_13 Ergebnis 0 \n");
				#endif
				loop = FALSE;
				break;
			}
			
			// Werte jetzt die Daten aus und rufe entsprechend der Bedeutung Funktionen
			// des ActiveX auf -> dadurch wird wiederum ein EVENT erzeugt, welches direkt
			// an die Applikation weitergeleitet wird
			#ifdef TRACE
			TRACE("ISR-Nr.: %d Id 0x%x\n",info.isrNummer,info.canIdentifier);
			#endif
			if (info.isrNummer && pC->iShowMessages)
			{
				double absTime,relTime;
				HIGHLOW x;
				x = Div64To32(info.timeStamp,pC->dwTimeMultiplier,pC->dwStartTime);
				absTime = (double)x.low/1000.;
				relTime = (double)(x.low-pC->dwOldTime)/1000.; 
				pC->dwOldTime = x.low;
				pC->dwDataLost += info.lost;

				if (
					(info.msg == EV2PRG_RECEIVED) || (info.msg == EV2PRG_TRANSMIT) || 
					(info.msg == EV2PRG_RECEIVED_STD) || (info.msg == EV2PRG_TRANSMIT_STD) ||
					(info.msg == EV2PRG_RECEIVED_EXT) || (info.msg == EV2PRG_TRANSMIT_EXT)
				   )
				{
					int bits=29;
					if(pC->iFormat)
						bits+=18;
					if(!info.canRTR)
						bits+=info.canDLC*8;
					
					EnterCriticalSection(&BusLoadCriticalSection);
					pC->statistikBits += bits;
					pC->statistikTime += relTime;
					LeaveCriticalSection(&BusLoadCriticalSection);
				}
				if(pC->events)
				{
					#ifdef TRACE
					TRACE("THREAD: Rufe User-Funktion auf %d\n",pC->iStopThread);
					#endif
					(*pC->events)(pC,absTime,relTime,&info);
					#ifdef TRACE
					TRACE("THREAD: User-Funktion fertig %d\n",pC->iStopThread);
					#endif
				}
			}

		}while(info.isrNummer && !pC->iStopThread);
	}
	#ifdef TRACE
	TRACE("THREAD: Wird beendet dwRet=0x%x\n",dwRet);
	#endif
	CancelIo(pC->hDevice);

	#ifdef TRACE
	TRACE("Schliee Treiber\n");
	#endif
	CloseHandle(pC->hDevice);
	pC->hDevice = NULL;

	#ifdef TRACE
	TRACE("Signal lschen\n");
	#endif
	CloseHandle(pC->hSignal);
	pC->hSignal = NULL;

	return 0;

}

static DWORD WINAPI canBusLoadThread(LPVOID lpThreadParam)
{
	struct can_api *pC = (struct can_api *)lpThreadParam;

	while(!pC->iStopThread)
	{
		if(pC->statistikTime > 0.)
		{
			double bps = pC->statistikBits/pC->statistikTime;
			pC->statistikBusload = bps * 100. / pC->dwBaudrate;
			EnterCriticalSection(&BusLoadCriticalSection);
			pC->statistikTime = 0.;
			pC->statistikBits = 0;
			LeaveCriticalSection(&BusLoadCriticalSection);
			#ifdef TRACE
			TRACE("Buslast: %03lf\n",pC->statistikBusload);
			#endif
		}
		else
			pC->statistikBusload = 0.;
		Sleep(1000);
	}
	#ifdef TRACE
	TRACE("BusLoad Thread wird beendet\n");
	#endif
	return 0;

}


struct can_api * WINAPI canInit(HWND hParent, const void *pUserData,int (*events)(struct can_api* c, double absTime, double relTime, ISRInfo *i))
{
	struct can_api *ret;
	ret=(struct can_api *)malloc(sizeof(struct can_api));
	if(ret)
	{
		InitializeCriticalSection(&BusLoadCriticalSection);
		memset(ret,0,sizeof(struct can_api));
		ret->hWnd = hParent;

		ret->pUserData = (void*)pUserData;
		ret->dwAccessCode= 0xffffffff;	// diese Werte sind DEFAULT im Treiber eingestellt
		ret->dwAccessMask = 0xffffffff;
		ret->dwBaudrate= 20000;
		ret->events = events;
		ret->iShowMessages = 1; // Default: Daten an Applikation senden
	}
	return ret;
}


void  WINAPI canDeInit(struct can_api *c)
{
	if(c)
	{
		canCloseDevice(c);
		DeleteCriticalSection(&BusLoadCriticalSection);
		free(c);
	}
}

int  WINAPI canOpenDevice(struct can_api *c, const char *pDevName, int mode, int format, DWORD *pErr) 
{
	unsigned char tmp;
	DWORD dwRet;

	if(!c) return 0;

	if (c->hDevice)
	{
		#ifdef TRACE
		TRACE("OpenCan wurde rekursiv aufgerufen -> Schliee zuerst den Treiber\n");
		#endif
		canCloseDevice(c);
	}

	c->iStopThread = 0;
// unter NT4 funktioniert folgende Zeile nicht, wenn man als "normaler" Anwender eingeloggt ist
//	c->hDevice = CreateFile(pDevName, 0,0,0,CREATE_NEW, FILE_FLAG_OVERLAPPED+FILE_FLAG_DELETE_ON_CLOSE, 0);
	c->hDevice = CreateFile(pDevName, 0,0,0,CREATE_NEW, FILE_FLAG_OVERLAPPED, 0);
	if (c->hDevice == INVALID_HANDLE_VALUE)
	{
		*pErr = GetLastError();
		#ifdef TRACE
		TRACE("CreateFile nicht OK\n");
		#endif
		canCloseDevice(c);
		return 0;
	}
	c->iMode = mode;
	c->iFormat = format;
	tmp = c->iMode;
	DeviceIoControl(c->hDevice, IOCTL_IO_14, &tmp, 1, NULL, 0, &dwRet, NULL);
	tmp = c->iFormat;
	DeviceIoControl(c->hDevice, IOCTL_IO_15, &tmp, 1, NULL, 0, &dwRet, NULL);
	
	c->hSignal = CreateEvent(NULL, FALSE, FALSE, NULL); // create auto-reset event
	if (!c->hSignal)			// Es konnte kein "SIGNAL" erzeugt werden
	{						// => d.h. der THREAD kann nicht benachrichtigt werden
		#ifdef TRACE
		TRACE("Signal kann nicht erzeugt werden\n");
		#endif
		*pErr = GetLastError();
		canCloseDevice(c);
		return -1;
	}
	
	DWORD threadId;
	c->hThread = CreateThread( NULL, 0, canBackgroundThread, c, 0 , &threadId );
	if (!c->hThread)
	{
		#ifdef TRACE
		TRACE("THREAD kann nicht erzeugt werden\n");
		#endif
		*pErr = GetLastError();
		canCloseDevice(c);
		return -2;
	}

	c->hBusLoadThread = CreateThread( NULL, 0, canBusLoadThread, c, 0 , &threadId );
	if (!c->hBusLoadThread)
	{
		#ifdef TRACE
		TRACE("THREAD kann nicht erzeugt werden\n");
		#endif
		*pErr = GetLastError();
		canCloseDevice(c);
		return -2;
	}

	memset(&c->info,0,sizeof(CANPARINFO));
	DeviceIoControl(c->hDevice, IOCTL_IO_1, NULL, 0, &c->info, sizeof(CANPARINFO), &dwRet, NULL);
	if (c->info.version != CANPAR_VERSION)
	{
		#ifdef TRACE
		TRACE("Falsche Treiberversion\n");
		#endif
		canCloseDevice(c);
		*pErr = 0;
		return -3;
	}

	switch(c->info.os)
	{
	default:
	case OS_WINDOWS9x:
			c->dwTimeMultiplier = TIME_DIVIDER_WIN9X;
			break;
	case OS_WINDOWSNT4:
			c->dwTimeMultiplier = TIME_DIVIDER_WINNT4;
			break;
	}
	
	HIGHLOW x = Div64To32(c->info.time,c->dwTimeMultiplier,0);
	c->dwOldTime = 0;
	c->dwStartTime = x.low;
	c->dwDataLost= 0;
	*pErr = 0;
	Sleep(200);
	return 1;
}

int  WINAPI canIsDeviceOpened(struct can_api *c) 
{
	if(!c) return 0;

	if (!c->hDevice)
		return 0;

	return 1;
}

void  WINAPI canCloseDevice(struct can_api *c) 
{
	if(!c) return;

    canDeleteAllTransmitThreads(c);

	c->iStopThread = TRUE;
	if (c->hSignal)
	{
		#ifdef TRACE
		TRACE("Sende Signal -> Thread soll beendet werden\n");
		#endif
		SetEvent(c->hSignal);
	}

	#ifdef TRACE
	TRACE("Warte auf Thread\n");
	#endif
	if (WAIT_TIMEOUT == WaitForSingleObject(c->hThread, 1000))
	{
		#ifdef TRACE
		TRACE("Timeout Thread\n");
		#endif
	}

	if (c->hThread)
	{
		#ifdef TRACE
		TRACE("Beende THREAD\n");
		#endif
		CloseHandle(c->hThread);
		c->hThread = NULL;
	}
	
	#ifdef TRACE
	TRACE("Warte auf BusLoad-Thread\n");
	#endif
	if (WAIT_TIMEOUT == WaitForSingleObject(c->hBusLoadThread, 1000))
	{
		#ifdef TRACE
		TRACE("Timeout BusLoad Thread\n");
		#endif
	}
	if (c->hBusLoadThread)
	{
		#ifdef TRACE
		TRACE("Beende BusLoadThread\n");
		#endif
		CloseHandle(c->hBusLoadThread);
		c->hBusLoadThread = NULL;
	}
}

double  WINAPI canGetBusLoad(struct can_api *c)
{
	if(!c) return 0.;
	return c->statistikBusload;
}

void  WINAPI canExtendedFrames(struct can_api *c, int format) 
{
	unsigned char tmp;
	DWORD dwRet;

	if(!c) return;
	c->iFormat = format;
	DeviceIoControl(c->hDevice, IOCTL_IO_15, &tmp, 1, NULL, 0, &dwRet, NULL);
}

unsigned char  WINAPI canGetRegisterRR1(struct can_api *c, unsigned char nIndex) 
{
	if (!c || !c->hDevice) return 0;

	unsigned char b;
	DWORD dwRet;
	DeviceIoControl(c->hDevice, IOCTL_IO_3, &nIndex,sizeof(nIndex), &b,1, &dwRet, NULL);
	return b;
}

void  WINAPI canSetRegisterRR1(struct can_api *c, unsigned char nIndex, unsigned char nNewValue) 
{
	if (!c || !c->hDevice) return;

	unsigned char b1[2],b2[128];
	DWORD dwRet;
	b1[0] = nIndex;
	b1[1] = nNewValue;
	DeviceIoControl(c->hDevice, IOCTL_IO_2, b1,sizeof(b1), b2,sizeof(b2), &dwRet, NULL);
}

unsigned char  WINAPI canGetRegister(struct can_api *c, unsigned char nIndex) 
{
	if (!c || !c->hDevice) return 0;

	unsigned char b;
	DWORD dwRet;
	DeviceIoControl(c->hDevice, IOCTL_IO_5, &nIndex,sizeof(nIndex), &b,1, &dwRet, NULL);
	return b;
}

void  WINAPI canSetRegister(struct can_api *c, unsigned char nIndex, unsigned char nNewValue) 
{
	if (!c || !c->hDevice) return;

	unsigned char b1[2],b2[128];
	DWORD dwRet;
	b1[0] = nIndex;
	b1[1] = nNewValue;
	DeviceIoControl(c->hDevice, IOCTL_IO_4, b1,sizeof(b1), b2,sizeof(b2), &dwRet, NULL);
}

int  WINAPI canTransmitDataByISR(struct can_api *c, struct ISRInfo *isr) 
{
	DWORD dwRet;

	if (!c || !c->hDevice) return 0;
	if (!DeviceIoControl(c->hDevice, IOCTL_IO_10,isr, sizeof(struct ISRInfo), NULL, 0, &dwRet, NULL))
	{
		int t = GetLastError();
		#ifdef TRACE
		TRACE("Fehler beim Senden der Daten: %d\n",t);
		#endif
		return t;
	}

	return 0;
}

int  WINAPI canTransmitData(struct can_api *c, unsigned long id, int rtr, int dlc, unsigned char* data) 
{
	struct ISRInfo isr;
	int t;

	if (!c || !c->hDevice) return 0;
	isr.canDLC = (unsigned char)dlc;
	isr.canIdentifier = id;
	isr.canRTR = (unsigned char)rtr;
	for(t=0;t<dlc;t++)
		isr.canMsg[t] = data[t];
	
	return canTransmitDataByISR(c, &isr);
}

unsigned char  WINAPI canGetSJA1000Status(struct can_api *c)
{
	char tmp;
	DWORD dwRet;
	if (!c || !c->hDevice) return 0;
	DeviceIoControl(c->hDevice, IOCTL_IO_8,NULL,0,&tmp, sizeof(tmp), &dwRet, NULL);
	return tmp;
}

int  WINAPI canSetSJA1000Command(struct can_api *c, unsigned char command) 
{
	char tmp;
	DWORD dwRet;
	if (!c || !c->hDevice) return 0;
	tmp = command;		// siehe CAN_COMMAND_... in canpar.h
	DeviceIoControl(c->hDevice, IOCTL_IO_16,&tmp, sizeof(tmp), NULL,0,&dwRet, NULL);
	return 1;
}

long  WINAPI canGetLostDataMsg(struct can_api *c)
{
	if (!c || !c->hDevice) return 0;
	return c->dwDataLost;
}

void  WINAPI canSetLostDataMsg(struct can_api *c, long nNewValue)
{
	if (!c || !c->hDevice) return;
	c->dwDataLost = nNewValue;
}

int  WINAPI canAcceptanceCodeMask(struct can_api *c, unsigned long code, unsigned long mask) 
{
	// Um den Acceptance Code in das SJA1000 zu schreiben
	// wird das ResetRequest-Bit gesetzt. Dadurch wird eine
	// laufende Botschaft abgebrochen. Um unntige Schreibvorgnge
	// zu verhindern, wird nur bei einer nderung der Schreibvorgang 
	// durchgefhrt.
	if (!c || !c->hDevice) return 0;
	if ((c->dwAccessCode != code) || (c->dwAccessMask != mask))
	{
		// Wert in den Treiber schreiben
		unsigned long tmp[2];
		DWORD dwRet;
		tmp[0] = code;
		tmp[1] = mask;
		c->dwAccessCode = code;
		c->dwAccessMask = mask;
		DeviceIoControl(c->hDevice, IOCTL_IO_9,tmp, sizeof(tmp), NULL,0,&dwRet, NULL);
		return 1;
	}
	return 0;
}

int  WINAPI canAcceptanceFilter(struct can_api *c, unsigned long code, unsigned long mask) 
{
	// Software Filter
	if (!c || !c->hDevice) return 0;

	// Wert in den Treiber schreiben
	unsigned long tmp[2];
	DWORD dwRet;
	tmp[0] = code;
	tmp[1] = mask;
	DeviceIoControl(c->hDevice, IOCTL_IO_17,tmp, sizeof(tmp), NULL,0,&dwRet, NULL);
	return 1;
}

int  WINAPI canSetBaudrate(struct can_api *c, unsigned long baudrate) 
{
	// Um die Baudrate in das SJA1000 zu schreiben
	// wird das ResetRequest-Bit gesetzt. Dadurch wird eine
	// laufende Botschaft abgebrochen. Um unntige Schreibvorgnge
	// zu verhindern, wird nur bei einer nderung der Schreibvorgang 
	// durchgefhrt.
	if (!c || !c->hDevice) return 0;
	if (c->dwBaudrate != baudrate)
	{
		DWORD dwRet;
		if(!DeviceIoControl(c->hDevice, IOCTL_IO_11,&baudrate, sizeof(baudrate), NULL,0,&dwRet, NULL))
		{
			// zuvor eingestellte Baudrate wieder einstellen
			DeviceIoControl(c->hDevice, IOCTL_IO_11,&c->dwBaudrate , sizeof(c->dwBaudrate ), NULL,0,&dwRet, NULL);
			return 0;
		}
		c->dwBaudrate = baudrate;
		return 1;
	}
	return 1;
}

void  WINAPI canResetTime(struct can_api *c)
{
	CANPARINFO info;
	DWORD dwRet;
	if (!c || !c->hDevice) return;
	memset(&info,0,sizeof(info));
	DeviceIoControl(c->hDevice, IOCTL_IO_1, NULL, 0, &info, sizeof(info), &dwRet, NULL);
	
	HIGHLOW x = Div64To32(info.time,c->dwTimeMultiplier,0);
	c->dwOldTime = 0;
	c->dwStartTime = x.low;
}

long  WINAPI canGetTime(struct can_api *c)
{
	CANPARINFO info;
	DWORD dwRet;
	if (!c || !c->hDevice) return 0;
	memset(&info,0,sizeof(info));
	DeviceIoControl(c->hDevice, IOCTL_IO_1, NULL, 0, &info, sizeof(info), &dwRet, NULL);
	HIGHLOW x = Div64To32(info.time,c->dwTimeMultiplier,0);
	return x.low;
}

void  WINAPI canShowMessages(struct can_api *c, int on_off)
{
	c->iShowMessages = (on_off)?TRUE:FALSE;
}

static struct can_api_send_thread * SearchTransmitThread(struct can_api *c, int threadId)
{
	struct can_api_send_thread *x;

	x = c->pThread;
	while(x)
	{
		if(x->id == threadId)
			return x;
		x = x->pNext;
	}
	return 0;
}

int  WINAPI canDeleteTransmitThread(struct can_api *c, int threadId) 
{
	struct can_api_send_thread *n;
	struct can_api_send_thread *x;
	HANDLE hTh;

	if (!c || !c->hDevice) return 0;

	#ifdef TRACE
	TRACE("canDeleteTransmitThread %d\n",threadId);
	#endif
	n = SearchTransmitThread(c, threadId);
	if(n)
	{
		// Lsche Struktur aus Liste
		if (c->pThread == n)
			c->pThread = n->pNext;
		else
		{
			x = c->pThread;
			while(x)
			{
				if (x->pNext == n)
				{
					x->pNext = n->pNext;
					break;
				}
				x = x->pNext;
			}
		}

		// STOP signalisieren und danach Thread HANDLE lschen
		hTh = n->hThread; // Merke mir Handle, da die Struktur im Thread zerstrt wird
		
		n->stop = 1; // Ab jetzt darf nirgends mehr auf 'n' zugegriffen werden!
		
		
		if (WAIT_TIMEOUT == WaitForSingleObject(n->hThread, 500))
		{
			#ifdef TRACE
			TRACE("Timeout SendeThread\n");
			#endif
		}
		
		if (hTh)
		{
			CloseHandle(hTh);
		}
		return 1;
	}
	return 0;
}

void  WINAPI canDeleteAllTransmitThreads(struct can_api *c)
{
	struct can_api_send_thread *x;
	x = c->pThread;
	while(x)
	{
		canDeleteTransmitThread(c, x->id);
		x = c->pThread;
	}
}

int  WINAPI canAddTransmitThread(struct can_api *c, int threadId, unsigned long canId, int rtr, int dlc, unsigned char *data, unsigned long delay) 
{
	struct can_api_send_thread *n,*x;

	if (!c || !c->hDevice) return 0;
	n = SearchTransmitThread(c, threadId);
	if(n)
	{
		// Thread bereits vorhanden -> nur neue Zeit eintragen
		n->dwDelay = delay;
		return 1;
	}

	// Thread-ID noch nicht vorhanden -> neu anlegen
	n = (struct can_api_send_thread *)malloc(sizeof(struct can_api_send_thread));
	if(n)
	{
		// Struktur angelegt -> init. jetzt Eintrge
		memset(n,0,sizeof(struct can_api_send_thread));
		n->dwDelay = delay;
		n->id = threadId;
		n->pMain = c;
		
		// zu sendende Botschaft
		for(int t=0;t<dlc;t++)
			n->isrCAN.canMsg[t] = data[t];
		n->isrCAN.canIdentifier=canId;
		n->isrCAN.canDLC=dlc;
		n->isrCAN.canRTR=rtr;

		// Thread anlegen
		DWORD threadId;
		n->hThread = CreateThread( NULL, 0, canSenderThread, n, 0 , &threadId );
		if (!n->hThread)
		{
			// Thread wurde nicht angelegt
			free(n);
			return 0;
		}

		// es knnen mehrere Threads angelegt werden -> in Liste anfgen
		if (!c->pThread)
		{
			c->pThread = n;
		}
		else
		{
			x = c->pThread;
			while(x->pNext) x = x->pNext;
			x->pNext = n;
		}
		return 1;
	}
	return 0;
}


//////////////////////////////////////////////////////////////////////////////////////////////////

#ifdef __cplusplus
}
#endif  /* __cplusplus */
