/*! \file canpar.h

  \par Header: 
  CANPAR.H

  \brief Strukturen und Konstanten

*/
/* Selfhtml im Netz: http://selfhtml.teamone.de/navigation/html.htm 

<center>
\htmlonly
<img align=center lowsrc="can-bus.jpg" src="can-bus.jpg" 
     width=634 height=197 ><br>
\endhtmlonly
</center> 

*/


/*! \mainpage CAN API Beschreibung $(VERSION)


\par Beschreibung:
Direktes einbinden in eigene Visual C++ Projekte

\par Header:
- CAN_API.H
- CANPAR.H

\section allgemein Allgemein:
Die Beschreibung soll helfen, die Treiberfunktionen direkt - also ohne das ActiveX - zu verwenden. 
Dies wird hiermit auch ausdrcklich empfohlen. Eine Lsung mit dem ActiveX wird immer etwas langsamer 
sein, als direkte API Aufrufe.

  Es gibt mehrere Mglichkeiten diese Routinen einzubinden:
- Direktes hinzufgen der CAN_API.CPP Datei in Visual C++ Projekte
- Verwenden der CAN_API.DLL, was auch in allen brigen Programmiersprachen funktionieren sollte, 
  getestet wurde diese DLL allerdings nur fr Visual C++ Programme (CATT.EXE)

Der Treiber beinhaltet alle Funktionen fr den BasicCAN und den PeliCAN Modus des SJA1000. 
Der BaiscCAN-Modus ist kompatibel zum PCA82c200, der daher auch mit diesem Treiber funktionieren  
sollte (wurde aber nicht getestet). 
blicherweise wird so vorgegangen:

-# Eine Callbackfunktion definieren, die durch den Treiber aufgerufen wird, wenn CAN Nachrichten oder 
   andere Ereignisse eintreffen. siehe   \link examples_canEvents.c Beispiel \endlink
-# Beim Programmstart wird canInit() aufgerufen. In canInit()  wird unter anderem die Callbackfunktion aus 1.) angegeben
-# CanOpenDevice() aufrufen. Dadurch wird das CAN Gert erst geffnet
-# CAN Nachrichten senden und empfangen und andere CAN API Funktionen aufrufen (z.B. Register auslesen, 
  beschreiben, ...)
-# CanCloseDevice() aufrufen. Das CAN Gert wird dadurch geschlossen
-# Vor Programmende dann canDeInit() aufrufen um die interne Strukturen wieder aufzurumen

*/


#if !CANPAR_FILE_HEADER_INCLUDE
#define CANPAR_FILE_HEADER_INCLUDE

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

/* CanPar - Treiber
   Diese Datei ist fr alle Betriebssysteme!
*/


/** \brief Version (zur Versionskontrolle)

  Die Nummer wird erhht wenn es zu inkompatibilitten kommt und dient zur Versionsberprfung
*/
#define CANPAR_VERSION 4






// DeviceIOControl-Codes fr NT4 und fr Win9x
// Dadurch kann das ActiveX unter beiden Betriebssystemen verwendet werden
#ifndef CAN_DRIVER_FOR_LINUX

#define FILE_DEVICE_CANPAR FILE_DEVICE_PARALLEL_PORT
#define IOCTL_IO_1   CTL_CODE(FILE_DEVICE_CANPAR, 0x801, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_IO_2   CTL_CODE(FILE_DEVICE_CANPAR, 0x802, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_IO_3   CTL_CODE(FILE_DEVICE_CANPAR, 0x803, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_IO_4   CTL_CODE(FILE_DEVICE_CANPAR, 0x804, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_IO_5   CTL_CODE(FILE_DEVICE_CANPAR, 0x805, METHOD_BUFFERED, FILE_ANY_ACCESS)
//#define IOCTL_IO_6   CTL_CODE(FILE_DEVICE_CANPAR, 0x806, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_IO_7   CTL_CODE(FILE_DEVICE_CANPAR, 0x807, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_IO_8   CTL_CODE(FILE_DEVICE_CANPAR, 0x808, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_IO_9   CTL_CODE(FILE_DEVICE_CANPAR, 0x809, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_IO_10  CTL_CODE(FILE_DEVICE_CANPAR, 0x810, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_IO_11  CTL_CODE(FILE_DEVICE_CANPAR, 0x811, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_IO_12  CTL_CODE(FILE_DEVICE_CANPAR, 0x812, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_IO_13  CTL_CODE(FILE_DEVICE_CANPAR, 0x813, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_IO_14  CTL_CODE(FILE_DEVICE_CANPAR, 0x814, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_IO_15  CTL_CODE(FILE_DEVICE_CANPAR, 0x815, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_IO_16  CTL_CODE(FILE_DEVICE_CANPAR, 0x816, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_IO_17  CTL_CODE(FILE_DEVICE_CANPAR, 0x817, METHOD_BUFFERED, FILE_ANY_ACCESS)

#else


#define CANPAR_IOC_MAGIC 'j'

#define IOCTL_IO_1   _IO(CANPAR_IOC_MAGIC, 1)
#define IOCTL_IO_2   _IO(CANPAR_IOC_MAGIC, 2)
#define IOCTL_IO_3   _IO(CANPAR_IOC_MAGIC, 3)
#define IOCTL_IO_4   _IO(CANPAR_IOC_MAGIC, 4)
#define IOCTL_IO_5   _IO(CANPAR_IOC_MAGIC, 5)
#define IOCTL_IO_7   _IO(CANPAR_IOC_MAGIC, 7)
#define IOCTL_IO_8   _IO(CANPAR_IOC_MAGIC, 8)
#define IOCTL_IO_9   _IO(CANPAR_IOC_MAGIC, 9)
#define IOCTL_IO_10  _IO(CANPAR_IOC_MAGIC, 10)
#define IOCTL_IO_11  _IO(CANPAR_IOC_MAGIC, 11)
#define IOCTL_IO_12  _IO(CANPAR_IOC_MAGIC, 12)
#define IOCTL_IO_13  _IO(CANPAR_IOC_MAGIC, 13)
#define IOCTL_IO_14  _IO(CANPAR_IOC_MAGIC, 14)
#define IOCTL_IO_15  _IO(CANPAR_IOC_MAGIC, 15)
#define IOCTL_IO_16  _IO(CANPAR_IOC_MAGIC, 16)


/* Interne Struktur im Treiber
  ber diese Struktur werden alle Parameter und Daten
  zwischen Applikation und Treiber ausgetauscht.
*/
struct ioctlTransfer
{
	void   *lpvInBuffer;			// Datenbergabe ber IOControl
	int		cbInBuffer;
	void   *lpvOutBuffer;
	int		cbOutBuffer;
	int		cbBytesReturned;
};
#endif




/** \defgroup CAN_COMMAND_ CAN_COMMAND_...Makros
 @{ 

  \brief Kommandos an den Treiber

  Die meisten Befehle werden sofort an den SJA1000 weitergeleitet (als Bitmuster in das
  dafr vorgesehene Command-Register) und von diesem dann erledigt.
*/
#define CAN_COMMAND_STANDBY 1                   /*!< Basic CAN Modus: Standby aktivieren */
#define CAN_COMMAND_ABORT_TRANSMISSION 2        /*!< Laufende bertragung abbrechen */
#define CAN_COMMAND_CLEAR_OVERRUN 3             /*!< Overrun Bit im SJA1000 rcksetzen */
#define CAN_COMMAND_STANDBY_OFF 4               /*!< Standby ausschalten */
#define CAN_COMMAND_DIFF_FRAMES_ON 5            /*!< Unterscheidung in Standard/Extended Frame Format einschalten */
#define CAN_COMMAND_DIFF_FRAMES_OFF 6           /*!< Unterscheidung in Standard/Extended Frame Format ausschalten */
// Verwendbar im PELICAN-CAN Modus
#define CAN_COMMAND_SELF_RECEPTION_REQUEST 0x80  /*!< Botschaft senden, die auch selbst wieder empfangen wird (fr Testzwecke) */
#define CAN_COMMAND_LISTEN_ON 0x81               /*!< Listen-Only Modus einschalten; SJA1000 sendet kein Acknowledge */
#define CAN_COMMAND_LISTEN_OFF 0x82              /*!< Listen-Only Modus ausschalten */
/** @} */






/* Strukturen die zwischen Applikation und Treiber verwendet werden */

/** \brief 64 bit Werte bergeben

  Struktur zur bergabe von 64 Bit Werte
*/
typedef struct HighLow  
{
	unsigned long low;    ///< untere 32 Bit
	unsigned long high;   ///< obere 32 Bit
}HIGHLOW;






/** \defgroup EV2PRG_ EV2PRG_...-Makros
 @{ 

  \brief Ereignisse

  Die auf dem CAN Bus auftretende Ereignisse werden ber die ISRInfo Struktur
  von der Applikation ausgelesen. Die Ereignisse werden ber diese
  Konstanten spezifiziert.
*/
#define EV2PRG_ERROR 1            /*!< (Veraltet) Dieses Ereignis wird nicht mehr verwendet! */
#define EV2PRG_RECEIVED 2         /*!< Daten wurden ber den CAN Bus empfangen */
#define EV2PRG_TRANSMIT 3         /*!< Daten wurden ber den CAN Bus gesendet */
#define EV2PRG_BUSERROR 4         /*!< Es sind Fehler auf dem Bus aufgetreten */
#define EV2PRG_OVERRUN 5          /*!< Es ist ein Overrun aufgetreten, d.h. eine unbestimmte Anzahl an CAN Botschaften konnte nicht abgespeichert werden*/
#define EV2PRG_WAKEUP 6           /*!< SJA1000 hat aus Standby zurck in den normalen Betrieb umgeschaltet */

// neu fr PELICAN-Modus
#define EV2PRG_WARNING 7          /*!< Warnung: Eine einstellbare Anzahl Fehler auf dem Bus hat diesen Interrupt ausgelst. \note Wird nur im PELICAN Modus akzeptiert */
#define EV2PRG_PASSIVE 8          /*!< SJA1000 hat Aufgrund von Busfehler in den Passivmodus umgeschaltet. \note Wird nur im PELICAN Modus akzeptiert */
#define EV2PRG_ARBITRATION 9      /*!< Der Sendevorgang wurde durch eine hher priorisierte Nachricht abgebrochen. \note Wird nur im PELICAN Modus akzeptiert */

// neu fr Demo
#define EV2PRG_BAUDRATE_DEMO 10   /*!< Die angegebene Baudrate kann im DEMOMODUS nicht eingestellt werden!  \note Wird nur im DEMO Modus ausgegeben. Die im Internet frei verfgbare Version ist eine DEMO Version! */

// neu zur Unterscheidung in Standard Frames und Extended Frames
#define EV2PRG_RECEIVED_STD 11    /*!< Die empfangene Nachricht wurde im Standard Frame Format (11 bit) empfangen */
#define EV2PRG_RECEIVED_EXT 12    /*!< Die empfangene Nachricht wurde im Extended Frame Format (29 bit) empfangen */
#define EV2PRG_TRANSMIT_STD 13    /*!< Die empfangene Nachricht wurde im Standard Frame Format (11 bit) gesendet */
#define EV2PRG_TRANSMIT_EXT 14    /*!< Die empfangene Nachricht wurde im Extended Frame Format (29 bit) gesendet */

/** @} */



/**\brief Datenaustausch zwischen Treiber und Applikation

  Der Treiber kann eine Applikation nur benachrichtigen dass Daten vorhanden sind (generieren eines Events),
  danach kann die Applikation die Daten abholen. Dies geschieht ber die ISRInfo Struktur.
*/
struct ISRInfo
{
	//unsigned long	refData;		// Parameter, der von der Applikation bergeben wurde (Zeiger auf Instanz der Klasse)
	unsigned long   msg;			//!< Inhalt, impliziert auch die gltigen Eintrge der Struktur \sa EV2PRG_
	HIGHLOW			timeStamp;		//!< Zeitmarke (64 Bit; Auflsung hngt vom Betriebssystem ab)
	unsigned long	canIdentifier;  //!< CAN Identifier, ob die ID im Standard Frame Format oder im Extended Frame Format angegeben wurde, ist aus dem EV2PRG_... Makros ersichtlich
	unsigned long	lost;			//!< Verlorene Botschaften (zwischen der aktuellen und der zuletzt in der Applikation empfangenen)
	unsigned char	canRTR;			//!< 1=Remote Transmission Request
	unsigned char	canDLC;			//!< Datenlnge; Anzahl der gltigen Bytes in canMsg[]
	unsigned char	status;			//!< Statusbyte aus SJA1000
	char			intern;			//!< \internal 1=Karte wurde rckgesetzt (dies behandelt der Treiber jetzt automatisch)
	short			isrNummer;		//!< Nummer zum freigeben der Struktur nach dem Auslesen
	unsigned char	canMsg[8];		//!< CAN-Botschaft; sie kann maximal 8 Bytes enthalten
};

typedef struct InitISRFunctions
{
	unsigned long	pCallback;
	unsigned long	refData;
}INITISRFUNCTIONS;



/** \defgroup OS_ OS_...-Makros
 @{ 
*/
#define OS_WINDOWS9x  1                /*!< Treiber fr Windows 9x \sa CanParInfo */
#define OS_WINDOWSNT4 2                /*!< Treiber fr Windows NT 4.0 \sa CanParInfo */
#define OS_WINDOWS2K  3                
										/*!< Treiber fr Windows 2000, da der NT4.0 Treiber auch unter
									         Windows 2000 arbeitet, wird dieser Wert nie verwendet \sa CanParInfo */
#define OS_LINUX      4                /*!< Treiber fr Linux \sa CanParInfo */
/** @} */


#define TIME_DIVIDER_WIN9X  (1250)     /*!< Win9x Auflsung: 800 ns */
#define TIME_DIVIDER_WINNT4 (1250*8)   /*!< Win NT 4.0 Auflsung: 100 ns */

/**\brief Treiberinformation auslesen

  Treiberinformation auslesen. In den CAN_API Routinen wird dies
  verwendet um:
  - die Version zu prfen
  - Startzeit feststellen (fr relative Zeiten)
*/
typedef struct CanParInfo
{
	unsigned long	version;		//!< Applikation und Treiber mssen zusammenpassen !! (Versionskontrolle)
	HIGHLOW			time;			//!< Zeitmarke (64-Bit Wert)
	unsigned char   card;			//!< 1=ELEKTOR parallel
	unsigned char   os;				//!< Betriebssystem \sa OS_
}CANPARINFO;

#endif
